@@ -16,6 +16,9 @@ from thrift.transport import TTransport
 from thrift.transport import TSocket
 from thrift.transport import THttpClient
 from thrift.protocol import TCompactProtocol
+import sys
+reload(sys)
+sys.setdefaultencoding("utf-8")
 
 #from curve import CurveThrift
 from curve import CurveThrift
@@ -29,11 +32,35 @@ __all__ = ['LineClient']
 
 EMAIL_REGEX = re.compile(r"[^@]+@[^@]+\.[^@]+")
 
+class LineContact(object):
+    """Contact object for LINE"""
+    def __init__(self, client, contact):
+        self._client  = client
+        self._contact = contact
+
+        self.mid           = contact.mid
+        self.displayName   = contact.displayName
+        self.statusMessage = contact.statusMessage
+
+    def sendMessage(self, text):
+        try:
+            self._client.sendMessage(id=self.mid, text=text)
+            return True
+        except Exception as e:
+            raise e
+
+    def __repr__(self):
+        return '<LineContact %s (%s)>' % (self.displayName, self.mid)
+
+    def __lt__(self, other):
+        return self.mid < other.mid
+
 class LineClient(object):
     """This class proviede you a way to communicate with LINE server.
 
-    :param email: NAVER id or NAVER email
-    :param password: LINE password
+        >>> client = LineClient(\'carpedm20\', \'xxxxxxxxxx\')
+        Enter PinCode '9779' to your mobile phone in 2 minutes
+        >>> print client.getProfile()
     """
 
     LINE_DOMAIN = "http://gd2.line.naver.jp"
@@ -48,6 +75,10 @@ class LineClient(object):
     version     = "3.7.0"
     com_name    = ""
 
+    contacts = []
+    rooms    = []
+    groups   = []
+
     _session = requests.session()
     _headers = {}
 
@@ -57,8 +88,8 @@ class LineClient(object):
         :param id: `NAVER id` or `LINE email`
         :param password: LINE account password
         :param authToken: LINE session key
-        :param is_mac: os setting
-        :param com_name: name of your system
+        :param is_mac: (optional) os setting
+        :param com_name: (optional) name of your system
         """
 
         if not (authToken or id and password):
@@ -97,13 +128,17 @@ class LineClient(object):
             self._login()
             self._ready()
 
+        self.refreshContacts()
+
     def raise_error(self, msg):
         raise Exception("Error: %s" % msg)
 
     def get_json(self, url):
+        """Get josn from given url with saved session and headers"""
         return json.loads(self._session.get(url, headers=self._headers).text)
 
     def check_auth(self):
+        """Check if client is logged in or not"""
         if self.authToken:
             return True
         else:
@@ -111,67 +146,97 @@ class LineClient(object):
             self.raise_error(msg)
 
     def getProfile(self):
+        """Get profile information
+        
+        :returns: Profile object
+                    - picturePath
+                    - displayName
+                    - phone (base64 encoded?)
+                    - allowSearchByUserid
+                    - pictureStatus
+                    - userid
+                    - mid # used for unique id for account
+                    - phoneticName
+                    - regionCode
+                    - allowSearchByEmail
+                    - email
+                    - statusMessage
+        """
         if self.check_auth():
             return self.client.getProfile()
 
+    def refreshContacts(self):
+        contact_ids = self.getAllContactIds()
+        contacts    = self.getContacts(contact_ids)
+
+        self.contacts = []
+
+        for contact in contacts:
+            self.contacts.append(LineContact(self, contact))
+
+        self.profile  = self.getProfile()
+        self.contacts.append(LineContact(self, self.profile))
+        self.contacts.sort()
+
     def getAllContactIds(self):
+        """Get all contacts of your LINE account"""
         if self.check_auth():
             return self.client.getAllContactIds()
 
-    def getRecommendationIds(self):
-        if self.check_auth():
-            return self.client.getRecommendationIds()
-
-    def getContacts(self, contact_list):
-        if type(contact_list) != list:
+    def getContacts(self, ids):
+        """Get contact information list from ids
+        
+        :returns: List of Contact list
+                    - status
+                    - capableVideoCall
+                    - dispalyName
+                    - settings
+                    - pictureStatus
+                    - capableVoiceCall
+                    - capableBuddy
+                    - mid
+                    - displayNameOverridden
+                    - relation
+                    - thumbnailUrl_
+                    - createdTime
+                    - facoriteTime
+                    - capableMyhome
+                    - attributes
+                    - type
+                    - phoneticName
+                    - statusMessage
+        """
+        if type(ids) != list:
             msg = "argument should be list of contact ids"
             raise_error(msg)
 
         if self.check_auth():
-            return self.client.getContacts(contact_list)
-
-    def getContactsName(self, contact_list):
-        if type(contact_list) != list:
-            msg = "argument should be list of contact ids"
-            raise_error(msg)
-
-        if self.check_auth():
-            contacts = self.client.getContacts(contact_list)
-
-            contact_list = []
-
-            for contact in contacts:
-                contact_dict = {}
-                contact_dict['mid'] = contact.mid
-                contact_dict['displayName'] = contact.displayName
-
-                contact_list.append(contact_dict)
-
-            return contact_list
+            return self.client.getContacts(ids)
 
     def getGroupIdsJoined(self):
+        """Get group id that you joined"""
         if self.check_auth():
             return self.client.getGroupIdsJoined()
 
-    def getGroups(self, group_list):
-        if type(group_list) != list:
+    def getGroups(self, ids):
+        if type(ids) != list:
             msg = "argument should be list of group ids"
             raise_error(msg)
 
         if self.check_auth():
-            return self.client.getGroups(group_list)
+            return self.client.getGroups(ids)
 
     def getRecentMessages(self, id, count=1):
         if self.check_auth():
             return self.client.getRecentMessages(id, count)
 
-    def sendMessage(self, id, text):
+    def sendMessage(self, id, text, seq=0):
         if self.check_auth():
             message = CurveThrift.Message(to=id,text=text)
 
-            return self.client.sendMessage(0, message)
+            return self.client.sendMessage(seq, message)
 
-    def getLastOpRevision():
+    def getLastOpRevision(self):
         if self.check_auth():
             return self.client.getLastOpRevision()
 
